==> spring-boot-ext-1.2.284-20190726.063757-33-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/replay/aop/ReplayRocketMqProducerPlugin.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.aop;

import cn.com.duibaboot.ext.autoconfigure.flowreplay.*;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.ReplayTraceContext;
import com.esotericsoftware.kryo.KryoException;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.Order;

import java.lang.reflect.Method;

/**
 * rocketmq回放切面
 * Created by guoyanfei .
 * 2019-05-23 .
 */
@Slf4j
@Aspect
@Order  // 默认最低优先级，这个拦截需要在最后，因为其他的拦截可能会在方法参数中加kv，如果再前面执行会导致回放参数对比不一致
public class ReplayRocketMqProducerPlugin {

    @Around("execution(* org.apache.rocketmq.client.producer.DefaultMQProducer+.*(..))")
    public Object rocketMqProducerJoinPoint(ProceedingJoinPoint joinPoint) throws Throwable {
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();

        // 当前不是回放的环境 或者 当前请求不是回放请求
        if (!FlowReplayUtils.isReplayEnv() || !ReplayTraceContext.isReplaying() || !"send".equals(method.getName())) {
            return joinPoint.proceed();
        }

        FlowReplaySpan span = ReplayTraceContext.pollSubSpan();

        log.debug("Rocketmq回放_traceId={}_spanType={}_spanId={}", ReplayTraceContext.getContextTraceId(), span != null ? span.getSpanType() : null, span != null ? span.getSpanId() : null);

        // 增加了新的调用 || 调用内容有变动
        if (span == null || SpanType.ROCKETMQ != span.getSpanType()) {
            String expert = SpanType.ROCKETMQ.name();
            String actual = span != null ? span.getSpanType().name() : null;
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_001, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        RocketMqProducerFlowReplaySpan rocketMqProducerSpan = (RocketMqProducerFlowReplaySpan) span;

        Object[] parameterValues = joinPoint.getArgs();

        // 方法名是否相同
        if (!FlowReplayUtils.isMethodEqual(rocketMqProducerSpan.getMethodName(), method)) {
            String expert = rocketMqProducerSpan.getMethodName();
            String actual = method.getName();
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1001, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        // 参数类型是否相同
        if (!FlowReplayUtils.isArgumentsTypesEqual(rocketMqProducerSpan.getParameterTypes(), method.getParameterTypes())) {
            String expert = FlowReplayUtils.stringArrayToString(rocketMqProducerSpan.getParameterTypes());
            String actual = FlowReplayUtils.classArrayToString(method.getParameterTypes());
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1002, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        try {
            // 参数值是否相同
            if (!FlowReplayUtils.isArgumentsEqual(rocketMqProducerSpan.getParameterValues(), parameterValues)) {
                ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1003);
                throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
            }
        } catch (KryoException e) {
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1006, e);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        // 返回值类型是否相同
        if (!FlowReplayUtils.isReturnTypeEqual(rocketMqProducerSpan.getReturnType(), method.getReturnType())) {
            String expert = rocketMqProducerSpan.getReturnType();
            String actual = method.getReturnType() != null ? method.getReturnType().getName() : null;
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1004, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        // 方法所在类全路径是否相同
        if (!FlowReplayUtils.isTypeFullPathEqual(rocketMqProducerSpan.getTypeFullPath(), signature.getDeclaringTypeName())) {
            String expert = rocketMqProducerSpan.getTypeFullPath();
            String actual = signature.getDeclaringTypeName();
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1005, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        try {
            // 都相同mock返回值
            return rocketMqProducerSpan.getReturnValue();
        } catch (KryoException e) {
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1007, e);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }
    }
}

==> spring-boot-ext-1.3.7.3-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/replay/aop/ReplayRocketMqProducerPlugin.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.aop;

import cn.com.duibaboot.ext.autoconfigure.flowreplay.*;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.replay.ReplayTraceContext;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.span.FlowReplaySpan;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.span.FlowReplayTrace;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.span.RocketMqProducerFlowReplaySpan;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.span.SpanType;
import com.esotericsoftware.kryo.KryoException;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.Order;

import java.lang.reflect.Method;

/**
 * rocketmq回放切面
 * Created by guoyanfei .
 * 2019-05-23 .
 */
@Slf4j
@Aspect
@Order  // 默认最低优先级，这个拦截需要在最后，因为其他的拦截可能会在方法参数中加kv，如果再前面执行会导致回放参数对比不一致
public class ReplayRocketMqProducerPlugin {

    @Around("execution(* org.apache.rocketmq.client.producer.DefaultMQProducer+.*(..))")
    public Object rocketMqProducerJoinPoint(ProceedingJoinPoint joinPoint) throws Throwable {   // NOSONAR
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();

        // 当前不是回放的环境 或者 当前请求不是回放请求
        if (!FlowReplayUtils.isReplayEnv() || !ReplayTraceContext.isReplaying() || !"send".equals(method.getName())) {
            return joinPoint.proceed();
        }

        FlowReplaySpan span = ReplayTraceContext.pollSubSpan();

        Object[] parameterValues = joinPoint.getArgs();
        // 创建回放的span，用于回放详情的构建
        RocketMqProducerFlowReplaySpan replayDetailSpan = createReplayDetailSpan(signature, parameterValues, span);

        // 增加了新的调用 || 调用内容有变动
        if (span == null || SpanType.ROCKETMQ != span.getSpanType()) {
            String expert = SpanType.ROCKETMQ.name();
            String actual = span != null ? span.getSpanType().name() : null;
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_001, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        RocketMqProducerFlowReplaySpan rocketMqProducerSpan = (RocketMqProducerFlowReplaySpan) span;

        // 方法名是否相同
        if (!FlowReplayUtils.isMethodEqual(rocketMqProducerSpan.getMethodName(), method)) {
            String expert = rocketMqProducerSpan.getMethodName();
            String actual = method.getName();
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1001, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        // 参数类型是否相同
        if (!FlowReplayUtils.isArgumentsTypesEqual(rocketMqProducerSpan.getParameterTypes(), method.getParameterTypes())) {
            String expert = FlowReplayUtils.stringArrayToString(rocketMqProducerSpan.getParameterTypes());
            String actual = FlowReplayUtils.classArrayToString(method.getParameterTypes());
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1002, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        try {
            // 参数值是否相同
            if (!FlowReplayUtils.isArgumentsEqual(rocketMqProducerSpan.getParameterValues(), parameterValues)) {
                ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1003);
                throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
            }
        } catch (KryoException e) {
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1006, e);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        // 返回值类型是否相同
        if (!FlowReplayUtils.isReturnTypeEqual(rocketMqProducerSpan.getReturnType(), method.getReturnType())) {
            String expert = rocketMqProducerSpan.getReturnType();
            String actual = method.getReturnType() != null ? method.getReturnType().getName() : null;
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1004, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        // 方法所在类全路径是否相同
        if (!FlowReplayUtils.isTypeFullPathEqual(rocketMqProducerSpan.getTypeFullPath(), signature.getDeclaringTypeName())) {
            String expert = rocketMqProducerSpan.getTypeFullPath();
            String actual = signature.getDeclaringTypeName();
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1005, expert, actual);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }

        try {
            // 都相同mock返回值
            Object mockRet = rocketMqProducerSpan.getReturnValue();

            // 参数都相同的情况下，mock之前把mock的返回值回填到构建回放详情用的span中
            replayDetailSpan.setReturnValue(mockRet);

            return mockRet;
        } catch (KryoException e) {
            ReplayTraceContext.markError(FlowReplayErrorMsgTypeEnum.EM_1007, e);
            throw new FlowReplayException(ReplayTraceContext.getCompletedErrorMsg());
        }
    }

    private RocketMqProducerFlowReplaySpan createReplayDetailSpan(MethodSignature signature, Object[] parameterValues, FlowReplaySpan span) {
        RocketMqProducerFlowReplaySpan replayDetailSpan = RocketMqProducerFlowReplaySpan.createSpan(signature, parameterValues);
        replayDetailSpan.setTraceId(FlowReplayTrace.getCurrentTraceId());
        if (span != null) {
            replayDetailSpan.setSpanId(span.getSpanId());
        }
        FlowReplayTrace.addSubSpan(replayDetailSpan);
        return replayDetailSpan;
    }
}
